unit GetWindow;

interface

uses Windows, Forms;

type
    TWindowCatcher = class
       public
        TargetHWND:LongInt;
        PointMode:Boolean;
        constructor Create(OwnerForm:TForm);
        procedure HighLightWindow(hWnd:LongInt);
        procedure StartCapture;
        procedure Move(X,Y: Integer);
        function  EndCapture:LongInt;
    end;

implementation
var FrmOwner:TForm;

constructor TWindowCatcher.Create(OwnerForm:TForm);
begin
     TargetHWND:=0;
     PointMode:=False;
     FrmOwner:=OwnerForm;
end;

procedure TWindowCatcher.HighLightWindow(hWnd:LongInt);
var
FrameCX,FrameCY,ScreenCX,ScreenCY,BorderCX,
hBrushOld,hPen,hPenOld,hdcDest:LongInt;
rctArea:TRect;
begin
     FrameCX:=GetSystemMetrics(SM_CXFRAME);    //32 Resizable Window Frame Width
     FrameCY:=GetSystemMetrics(SM_CYFRAME);    //33 --        --     --    Height
     ScreenCX:=GetSystemMetrics(SM_CXSCREEN);  //0  Screen Width
     ScreenCY:=GetSystemMetrics(SM_CYSCREEN);  //1  --     Height
     BorderCX:=GetSystemMetrics(SM_CXBORDER);  //5 Border Width

     hdcDest:=GetWindowDC(hWnd);
     //We shall draw an Inverse frame:
     SetROP2(hDCDest,R2_NOT);
     //Create a "NOT" pen
     hPen:=CreatePen(PS_INSIDEFRAME,3*BorderCX,0);
     hPenOld:=SelectObject(hdcDest,hPen);
     //No painting inside the window, select NULL_BRUSH
     hBrushOld:=SelectObject(hdcDest,GetStockObject(NULL_BRUSH));

     //Draw the Frame!
     If IsZoomed(hWnd) then
         Rectangle(hdcDest, FrameCX, FrameCY, ScreenCX+FrameCX, ScreenCY+FrameCY)
     Else
         begin
         GetWindowRect(hWnd, rctArea);
         Rectangle(hDcDest, 0, 0, rctArea.Right - rctArea.Left, rctArea.Bottom - rctArea.Top);
     end;


     //Restore previous Pen & Brush
     SelectObject(hdcDest, hBrushOld);
     SelectObject(hdcDest, hPenOld);
     ReleaseDC(hWnd, hdcDest);
     DeleteObject(hPen);
end;


procedure TWindowCatcher.StartCapture();
begin
     FrmOwner.Visible:=False;
     PointMode:=True;
     SetCapture(FrmOwner.Handle)
end;



procedure TWindowCatcher.Move(X, Y: Integer);
var
pt:TPoint;
hWndCur:LongInt;

begin
     if PointMode = False then Exit;
     pt.x:=x;
     pt.y:=y;
     //Convert Form-Relative Coords to Screen-Relative Coords
     Windows.ClientToScreen(FrmOwner.Handle,pt);
     //Get some Window, that is at that position
     hWndCur:=WindowFromPoint(pt);

     if TargetHWND <> hWndCur then begin
         //if this is not the first time we call the proc...

         //We draw with INVERSE pen, so to remove the frame from the last window
         //just call the function 2nd time on it
         if TargetHWND <> 0  then HighlightWindow(TargetHWND);
         //Make the New window selected
         HighLightWindow(hWndCur);
         TargetHWND:=hWndCur;
     end;
end;

function TWindowCatcher.EndCapture:LongInt;
begin
     PointMode:=False;
     HighlightWindow(TargetHWND);
     EndCapture:=TargetHWND;
     TargetHWND:=0;
     FrmOwner.Visible:=True;

end;

//2000KT, kt_ee@hotmail.com, http://smartsite.cjb.net
end.
