unit KTWindow;{Version 1.00}                
{This unit contains the TWindow class,
that makes operation with windows easier to write and understand
( e.g. instead of writing ShowWindowAsync(hWnd,SW_MAXIMIZE) you will
  write SomeWindow.Maximized:=True;)

  It is surprising, that I didn't find any analogue neither in Delphi files,
  nor anywhere else, so I had to make one.

  Of course, it is very easy to operate windows without it,
  but to my mind, such implementation is more 'beautiful'

Copyright (Alt+0169): 1999-2000, Konstantin Tretyakov
All Rights reserved.

You may use it freely.
If you think that I deserve something for typing all this,
 please, give me credit.

You may redistribute it in unmodified state
  (at least, don't modify the copyright message)
You may modify it, but if you do (modify, improve, find bugs),
    please, mail me (your modified variant or suggestion):

    kt_ee@hotmail.com
    http://smartsite.cjb.net
}


{Sample usage:
var wnd:TWindow;
wnd:=TWindow.Create;   //Create class, referencing no window
wnd.Handle:=wnd.FindWindow('CabinetWClass',nil);  //Some functions in the class are just API copies;
wnd.EnumFlags:=KTEW_GETTEXTS or KTEW_GETCLASSES or
   KTEW_ENUMVISIBLE or KTEW_ENUMINVISIBLE;
    //Means that when enumerating we want to get all windows
    //(visible & invisible) and "on the fly" get their names and classes
wnd.EnumChildWindows;   //You may also make: SomeHwndArray:=wnd.EnumChildWindows, but
    //Variable HwndArray (declared in the unit) will be anyway automatically filled;
    //WindowClasses and WindowTexts string lists will be filled,
    //if you specify KTEW_GETCLASSES and KTEW_GETTEXTS in the EnumFlags property respectively
//Here we can get the Handle of any window, its Caption/Text and Class like this:
//HwndArray[Number], WindowTexts[Number], WindowClasses[Number], where
// 0<=Number<=Length(HwndArray)-1
}
{For further help, see the comments in the class interface section
}

interface
uses Windows,Messages,SysUtils {for StrAlloc and Exceptions},Classes {for TStringList};

type
    TWindowProc = function(hWnd:HWND;Msg:Cardinal;wParam,lParam:Integer):Cardinal;stdcall;
    THwndArray=array of HWND;
    PHwndArray=^THwndArray;
    TWindowClass=class
    protected
        Handle:HWND;
    private
        function GetCLng(Index:Integer):Cardinal;
        procedure SetCLng(Index:Integer;NewVal:Cardinal);
        function GetWindowProc:TWindowProc;
        procedure SetWindowProc(NewProc:TWindowProc);
        //function GetCMenu:String;
        //procedure SetCMenu(NewVal:String);
    public
        constructor Create(NewHwnd:HWND);
        property Icon:Cardinal index GCL_HICON read GetCLng write SetCLng;      //I was just bored to make these of HICON Type
        property IconSmall:Cardinal index GCL_HICONSM read GetCLng write SetCLng;//Anyway, it works so not better, than if it were HICON
        property Cursor:Cardinal index GCL_HCURSOR read GetCLng write SetCLng;
        property Module:Cardinal index GCL_HMODULE read GetCLng write SetCLng;
        property BackgroundBrush:Cardinal index GCL_HBRBACKGROUND read GetCLng write SetCLng;
        property Atom:Cardinal index GCW_ATOM read GetCLng write SetCLng;
        property Style:Cardinal index GCL_STYLE	read GetCLng write SetCLng;
        property ClassExtraMemory:Cardinal index GCL_CBCLSEXTRA	read GetCLng write SetCLng;
        property WindowExtraMemory:Cardinal index GCL_CBWNDEXTRA read GetCLng write SetCLng;
        //property MenuName:String read GetCMenu write SetCMenu;
        property WindowProc:TWindowProc read GetWindowProc write SetWindowProc;
        //function ClassInfo:tagWNDCLASSEXA; //This one duzn't work, I don't know why
        function Name:String;
        function RegisterClassEx(WndClass:TWndClassEx):ATOM;
        function UnregisterClass(lpClassName:PChar;hInstanc:HINST):Boolean;
    end;
    TWindow = class(TObject)
    private
      FHandle:HWND;
      FEnumFlags:Cardinal;
      function GetText:String;
      procedure SetText(NewText:String);
      function GetWndLong(Index:Integer):Integer;
      procedure SetWndLong(Index:Integer;NewStyle:Integer);
      function GetHINSTANCE:HINST;
      procedure SetHINSTANCE(NewHINSTANCE:HINST);
      function GetWindowProc:TWindowProc;
      procedure SetWindowProc(NewWindowProc:TWindowProc);
      function GetPrnt:HWND;
      procedure SetPrnt(NewParent:HWND);
      function GetEnabled:Boolean;
      procedure SetEnabled(NewVal:Boolean);
      function GetWindowRect:TRect;
      procedure SetWindowRect(NewRect:TRect);
      function GetCoord(Index:Integer):Integer;
      procedure SetCoord(Index:Integer;NewVal:Integer);
      function GetVisible:Boolean;
      procedure SetVisible(NewVal:Boolean);
      function GetMinimized:Boolean;
      procedure SetMinimized(NewVal:Boolean);
      function GetMaximized:Boolean;
      procedure SetMaximized(NewVal:Boolean);
      function GetRestored:Boolean;
      procedure SetRestored(NewVal:Boolean);
      //function GetTopmost:Boolean;       //HOW ??
      procedure SetTopmost(NewVal:Boolean);
      function GetMenu:HMenu;
      procedure SetMenu(NewMenu:HMenu);
      function GetIcon(Index:Integer):HIcon;
      procedure SetIcon(Index:Integer;NewIcon:HIcon);
      procedure SetHandle(NewVal:HWND);
    public
      WindowClass:TWindowClass;
      property Handle:HWND read FHandle write SetHandle;
      constructor Create(HWND:HWND=0);
      function GetDesktopWindow:HWND;
      function GetActiveWindow:HWND;                 //Active Window here means the window, active for the CURRENT THREAD
      function SetActive:HWND; //if the Window belongs to a thread, running in background then it WILL become active for its thread, but will not become FOREGROUND; the result is the previous active window of the thread
      function GetForegroundWindow:HWND;
      function SetForeground:Boolean;   //if fails, returns False
      function GetHighestWindow:HWND;  //Often it is a TaskBar
      function GetLowestWindow:HWND;
      function TopChild:HWND;
      function Owner:Hwnd;
      function GetWindowAbove:HWND;
      function GetWindowBelow:HWND;
      function GetWindowFromDC(hDC:HDC):HWND;
      function GetWindowFromPoint(pt:TPoint):HWND;
      function ChildFromPoint(pt:TPoint):HWND;  //pt is relative to parent's clientarea, if pt is outside of the parent window, function returns 0, if it is inside parent, but not inside any child, it returns ParentHwnd, else it returns top child Hwnd, even if the child is Hidden/disabled
      function ChildFromPointEx(pt:TPoint;Flags:Cardinal):HWND;//Flags are: CWP_ALL CWP_SKIPINVISIBLE CWP_SKIPDISABLED CWP_SKIPTRANSPARENT
      function BringToFront:Boolean;
      procedure DestroyWindow;      //This one doesn't return, 'cause it may take too long for the window to destroy itself
      function Close:Boolean;       //This one only minimizes the window
      property Text:String read GetText write SetText;
      property Style:Integer index GWL_STYLE read GetWndLong write SetWndLong;
      property ExStyle:Integer index GWL_EXSTYLE read GetWndLong write SetWndLong;
      property UserData:Integer index GWL_USERDATA read GetWndLong write SetWndLong;
      property ID:Integer index GWL_ID read GetWndLong write SetWndLong;
      property HINSTANCE:HINST read GetHINSTANCE write SetHINSTANCE;
      property WindowProc:TWindowProc read GetWindowProc write SetWindowProc;
      property Parent:HWND read GetPrnt write SetPrnt;
      property Enabled:Boolean read GetEnabled write SetEnabled;
      property WindowRect:TRect read GetWindowRect write SetWindowRect;
      property Left:Integer index 0 read GetCoord write SetCoord;
      //property Right:Integer index 1 read GetCoord write SetCoord;    //Uncomment these-and they will work, (however look at them to understand what they do)
      property Top:Integer index 2 read GetCoord write SetCoord;
      //property Bottom:Integer index 3 read GetCoord write SetCoord;
      property Width:Integer index 4 read GetCoord write SetCoord;
      property Height:Integer index 5 read GetCoord write SetCoord;
      property Visible:Boolean read GetVisible write SetVisible;
      property Minimized:Boolean read GetMinimized write SetMinimized;   //Setting these properties occasionally shows the window (if it was hidden)
      property Maximized:Boolean read GetMaximized write SetMaximized;   //use the TWindow.Show function for more options (that is direct copy of Windows.ShowWindowAsync)
      property Restored:Boolean read GetRestored write SetRestored;
      property AlwaysOnTop:Boolean  write SetTopmost; //Write-Only, doesn't activate the window
      property Menu:HMenu read GetMenu write SetMenu;
      property Icon:HIcon index ICON_BIG read GetIcon write SetIcon;  //Big Icon is the one, shown in the TaskBar  //However, It works better, if you use TWindow.WindowClass.Icon
      property IconSmall:HIcon index ICON_SMALL read GetIcon write SetIcon;  //Small Icon is the one in the TitleBar
      function Valid:Boolean;
      function IsUnicode:Boolean;
      function ThreadID:Cardinal;
      function ProcessID:Cardinal;
      function ClassName:String;   //Same as TWindow.WindowClass.Name
      function ClientHDC:HDC;     //You must ReleaseDC(hDC) after you use it
      function WindowHDC:HDC;
      function ClientRect:TRect;  //Client rect's Left and top are always 0
      function TextLength:Integer;
      function IsChildOf(ParentHwnd:HWND):Boolean;
      procedure Repaint;
      function Move(Left,Top,Width,Height:Integer;Repaint:Boolean):Boolean;
      property EnumFlags:Cardinal read FEnumFlags write FEnumFlags; //Default is (KTEW_GETTITLES or KTEW_GETCLASSES), that means, that along with filling HwndArray array, function will also fill the WindowTexts and WindowClasses arrays
      function EnumChildWindows:THwndArray;
      function ClientToScreen(pt:TPoint):TPoint;   //These are copies of such WinAPI functions
      function EnumWindows:THwndArray;
      function EnumThreadWindows(dwThreadID:Cardinal):THwndArray;
      function ScreenToClient(pt:TPoint):TPoint;
      function FindWindow(lpClass,lpTitle:PChar):HWND;
      function FindWindowEx(hwndParent,hwndChildAfter:HWND;lpClass,lpTitle:PChar):HWND;
      function Show(CmdShow:Integer):Boolean; {CmdShow may be:SW_HIDE,SW_MAXIMIZE,SW_MINIMIZE,SW_RESTORE,SW_SHOW,SW_SHOW,SW_SHOWDEFAULT,SW_SHOWMAXIMIZED,SW_SHOWMINIMIZED,SW_SHOWMINNOACTIVE,SW_SHOWNA,SW_SHOWNOACTIVATE,SW_SHOWNORMAL}
      function PostMessage(Msg:Cardinal;wParam,lParam:Integer):Boolean;
      function SendMessage(Msg:Cardinal;wParam,lParam:Integer):Integer;
      function CreateWindowEx(dwExStyle: DWORD; lpClassName: PChar;
  lpWindowName: PChar; dwStyle: DWORD; X, Y, nWidth, nHeight: Integer;
  hWndParent: HWND; hMenu: HMENU; hInstance: HINST; lpParam: Pointer): HWND;
     function ListItemCount:Integer;  //For Working with a ListBox or a ComboBox;
     function ListItems:TStringList;  //Returns a list of strings in a ListBox or a ComboBox;
  end;
var
   HwndArray:THwndArray;
   WindowTexts:TStringList;
   WindowClasses:TStringList;
const
   KTEW_GETTEXTS=1;
   KTEW_GETCLASSES=2;
   KTEW_ENUMVISIBLE=4;
   KTEW_ENUMINVISIBLE=8;
   KTEW_DEFAULT=15;
{Window's ExStyle is a combination of:
WS_EX_ACCEPTFILES	Specifies that a window created with this style accepts drag-drop files.
WS_EX_APPWINDOW	Forces a top-level window onto the taskbar when the window is minimized.
WS_EX_CLIENTEDGE	Specifies that a window has a border with a sunken edge.
WS_EX_CONTEXTHELP	Includes a question mark in the title bar of the window. When the user clicks the question mark, the cursor changes to a question mark with a pointer. If the user then clicks a child window, the child receives a WM_HELP message. The child window should pass the message to the parent window procedure, which should call the WinHelp function using the HELP_WM_HELP command. The Help application displays a pop-up window that typically contains help for the child window.WS_EX_CONTEXTHELP cannot be used with the WS_MAXIMIZEBOX or WS_MINIMIZEBOX styles.
WS_EX_CONTROLPARENT	Allows the user to navigate among the child windows of the window by using the TAB key.
WS_EX_DLGMODALFRAME	Creates a window that has a double border; the window can, optionally, be created with a title bar by specifying the WS_CAPTION style in the dwStyle parameter.
WS_EX_LEFT	Window has generic "left-aligned" properties. This is the default.
WS_EX_LEFTSCROLLBAR	If the shell language is Hebrew, Arabic, or another language that supports reading order alignment, the vertical scroll bar (if present) is to the left of the client area. For other languages, the style is ignored and not treated as an error.
WS_EX_LTRREADING	The window text is displayed using Left to Right reading-order properties. This is the default.
WS_EX_MDICHILD	Creates an MDI child window.
WS_EX_NOPARENTNOTIFY	Specifies that a child window created with this style does not send the WM_PARENTNOTIFY message to its parent window when it is created or destroyed.
WS_EX_OVERLAPPEDWINDOW	Combines the WS_EX_CLIENTEDGE and WS_EX_WINDOWEDGE styles.
WS_EX_PALETTEWINDOW	Combines the WS_EX_WINDOWEDGE, WS_EX_TOOLWINDOW, and WS_EX_TOPMOST styles.
WS_EX_RIGHT	Window has generic "right-aligned" properties. This depends on the window class. This style has an effect only if the shell language is Hebrew, Arabic, or another language that supports reading order alignment; otherwise, the style is ignored and not treated as an error.
WS_EX_RIGHTSCROLLBAR	Vertical scroll bar (if present) is to the right of the client area. This is the default.
WS_EX_RTLREADING	If the shell language is Hebrew, Arabic, or another language that supports reading order alignment, the window text is displayed using Right to Left reading-order properties. For other languages, the style is ignored and not treated as an error.
WS_EX_STATICEDGE	Creates a window with a three-dimensional border style intended to be used for items that do not accept user input.
WS_EX_TOOLWINDOW	Creates a tool window; that is, a window intended to be used as a floating toolbar. A tool window has a title bar that is shorter than a normal title bar, and the window title is drawn using a smaller font. A tool window does not appear in the taskbar or in the dialog that appears when the user presses ALT+TAB.
WS_EX_TOPMOST	Specifies that a window created with this style should be placed above all non-topmost windows and should stay above them, even when the window is deactivated. To add or remove this style, use the SetWindowPos function.
WS_EX_TRANSPARENT	Specifies that a window created with this style is to be transparent. That is, any windows that are beneath the window are not obscured by the window. A window created with this style receives WM_PAINT messages only after all sibling windows beneath it have been updated.
WS_EX_WINDOWEDGE}
{Window's Style property:
WS_BORDER	Creates a window that has a thin-line border.
WS_CAPTION	Creates a window that has a title bar (includes the WS_BORDER style).
WS_CHILD	Creates a child window. This style cannot be used with the WS_POPUP style.
WS_CHILDWINDOW	Same as the WS_CHILD style.
WS_CLIPCHILDREN	Excludes the area occupied by child windows when drawing occurs within the parent window. This style is used when creating the parent window.
WS_CLIPSIBLINGS	Clips child windows relative to each other; that is, when a particular child window receives a WM_PAINT message, the WS_CLIPSIBLINGS style clips all other overlapping child windows out of the region of the child window to be updated. If WS_CLIPSIBLINGS is not specified and child windows overlap, it is possible, when drawing within the client area of a child window, to draw within the client area of a neighboring child window.
WS_DISABLED	Creates a window that is initially disabled. A disabled window cannot receive input from the user.
WS_DLGFRAME	Creates a window that has a border of a style typically used with dialog boxes. A window with this style cannot have a title bar.
WS_GROUP	Specifies the first control of a group of controls. The group consists of this first control and all  controls defined after it, up to the next control with the WS_GROUP style. The first control in each group usually has the WS_TABSTOP style so that the user can move from group to group. The user can subsequently change the keyboard focus from one control in the group to the next control in the group by using the direction keys.
WS_HSCROLL	Creates a window that has a horizontal scroll bar.
WS_ICONIC	Creates a window that is initially minimized. Same as the WS_MINIMIZE style.
WS_MAXIMIZE	Creates a window that is initially maximized.
WS_MAXIMIZEBOX	Creates a window that has a Maximize button. Cannot be combined with the WS_EX_CONTEXTHELP style. The WS_SYSMENU style must also be specified.
WS_MINIMIZE	Creates a window that is initially minimized. Same as the WS_ICONIC style.
WS_MINIMIZEBOX	Creates a window that has a Minimize button. Cannot be combined with the WS_EX_CONTEXTHELP style. The WS_SYSMENU style must also be specified.
WS_OVERLAPPED	Creates an overlapped window. An overlapped window has a title bar and a border. Same as the WS_TILED style.
WS_OVERLAPPEDWINDOW	Creates an overlapped window with the WS_OVERLAPPED, WS_CAPTION, WS_SYSMENU, WS_THICKFRAME, WS_MINIMIZEBOX, and WS_MAXIMIZEBOX styles. Same as the WS_TILEDWINDOW style. 
WS_POPUP	Creates a pop-up window. This style cannot be used with the WS_CHILD style.
WS_POPUPWINDOW	Creates a pop-up window with WS_BORDER, WS_POPUP, and WS_SYSMENU styles. The WS_CAPTION and WS_POPUPWINDOW styles must be combined to make the window menu visible.
WS_SIZEBOX	Creates a window that has a sizing border. Same as the WS_THICKFRAME style.
WS_SYSMENU	Creates a window that has a window-menu on its title bar. The WS_CAPTION style must also be specified.
WS_TABSTOP	Specifies a control that can receive the keyboard focus when the user presses the TAB key. Pressing the TAB key changes the keyboard focus to the next control with the WS_TABSTOP style.
WS_THICKFRAME	Creates a window that has a sizing border. Same as the WS_SIZEBOX style.
WS_TILED	Creates an overlapped window. An overlapped window has a title bar and a border. Same as the WS_OVERLAPPED style.
WS_TILEDWINDOW	Creates an overlapped window with the WS_OVERLAPPED, WS_CAPTION, WS_SYSMENU, WS_THICKFRAME, WS_MINIMIZEBOX, and WS_MAXIMIZEBOX styles. Same as the WS_OVERLAPPEDWINDOW style.
WS_VISIBLE	Creates a window that is initially visible.
WS_VSCROLL	Creates a window that has a vertical scroll bar.
}

implementation

constructor TWindow.Create(HWND:HWND);
begin
     inherited Create;
     FHandle:=HWND;
     FEnumFlags:=KTEW_GETTEXTS or KTEW_GETCLASSES or KTEW_ENUMVISIBLE or KTEW_ENUMINVISIBLE;
     if WindowTexts=nil then WindowTexts:=TStringList.Create;
     if WindowClasses=nil then WindowClasses:=TStringList.Create;
     WindowClass:=TWindowClass.Create(FHandle);
end;
procedure TWindow.SetHandle(NewVal:HWND);
begin
     FHandle:=NewVal;
     WindowClass.Handle:=NewVal;
end;
function TWindow.GetDesktopWindow:HWND;
begin
     Result:=GetDesktopWindow;
end;
function TWindow.GetActiveWindow:HWND;
begin
     Result:=Windows.GetActiveWindow;
end;
function TWindow.SetActive:HWND;
begin
     Result:=Windows.SetActiveWindow(Handle);
end;
function TWindow.GetForegroundWindow:HWND;
begin
     Result:=Windows.GetForegroundWindow;
end;
function TWindow.SetForeground:Boolean;
begin
     Result:=Windows.SetForegroundWindow(Handle);
end;
function TWindow.TopChild:HWND;
begin
     Result:=Windows.GetTopWindow(Handle);
end;
function TWindow.Owner:Hwnd;
begin
     Result:=Windows.GetWindow(Handle,GW_OWNER);
end;
function TWindow.GetWindowAbove:HWND;
begin
     Result:=GetNextWindow(Handle,GW_HWNDPREV);
end;
function TWindow.GetWindowBelow:HWND;
begin
     Result:=GetNextWindow(Handle,GW_HWNDNEXT);
end;
function TWindow.GetHighestWindow:HWND;
begin
     Result:=Windows.GetWindow(Handle,GW_HWNDFIRST);
end;
function TWindow.GetLowestWindow:HWND;
begin
     Result:=Windows.GetWindow(Handle,GW_HWNDLAST);
end;
function TWindow.BringToFront:Boolean;
begin
     Result:=BringWindowToTop(Handle);
end;
procedure TWindow.DestroyWindow;
begin
     //Result:=Windows.DestroyWindow(Handle);      This must be used only for current thread's windows
     Windows.PostMessage(Handle,WM_CLOSE,0,0);   //PostMessage, and do not wait for the result
end;
function TWindow.Close:Boolean;
begin
     Result:=CloseWindow(Handle);
end;
function TWindow.TextLength:Integer;
begin
     Result:=GetWindowTextLength(Handle);
end;
procedure TWindow.SetText(NewText:String);
begin
     if SetWindowText(Handle,PChar(NewText)) = False then raise Exception.Create('Win32 API Error: SetWindowText');
end;
function TWindow.Valid:Boolean;
begin
     Result:=Windows.IsWindow(Handle);
end;
function TWindow.ThreadID:Cardinal;
begin
     Result:=GetWindowThreadProcessID(Handle,nil);
end;
function TWindow.ProcessID:Cardinal;
begin
     GetWindowThreadProcessID(Handle,@Result);
end;
function TWindow.IsUnicode:Boolean;
begin
     Result:=IsWindowUnicode(Handle);
end;
function TWindow.FindWindow(lpClass,lpTitle:PChar):HWND;
begin
     Result:=Windows.FindWindow(lpClass,lpTitle);
end;
function TWindow.FindWindowEx(hwndParent,hwndChildAfter:HWND;lpClass,lpTitle:PChar):HWND;
begin
     Result:=Windows.FindWindowEx(hwndParent,hwndChildAfter,lpClass,lpTitle);
end;
function TWindow.Show(CmdShow:Integer):Boolean;
begin
     Result:=Windows.ShowWindowAsync(Handle,CmdShow);
end;
function TWindow.GetWndLong(Index:Integer):Integer;
begin
     Result:=GetWindowLong(Handle,Index);
end;
procedure TWindow.SetWndLong(Index:Integer;NewStyle:Integer);
begin
     if SetWindowLong(Handle,Index,NewStyle)=0 then raise Exception.Create('Win32 API Error: SetWindowLong(GWL_STYLE)');
end;
function TWindow.GetHINSTANCE:HINST;
begin
     Result:=GetWindowLong(Handle,GWL_HINSTANCE);
end;
procedure TWindow.SetHINSTANCE(NewHINSTANCE:HINST);
begin
     if SetWindowLong(Handle,GWL_HINSTANCE,NewHINSTANCE)=0 then raise Exception.Create('Win32 API Error: SetWindowLong(GWL_HINSTANCE)');
end;
function TWindow.GetWindowProc:TWindowProc;
begin
     @Result:=Pointer(GetWindowLong(Handle,GWL_WNDPROC));
end;
procedure TWindow.SetWindowProc(NewWindowProc:TWindowProc);
begin
     if SetWindowLong(Handle,GWL_WNDPROC,Integer(@NewWindowProc))=0 then raise Exception.Create('Win32 API Error: SetWindowLong(GWL_WNDPROC)');
end;
function TWindow.GetPrnt:HWND;
begin
     Result:=Windows.GetParent(Handle);
end;
procedure TWindow.SetPrnt(NewParent:HWND);
begin
     if Windows.SetParent(Handle,NewParent) = 0 then raise Exception.Create('Win32 API Error: SetParent');
end;
function TWindow.GetEnabled:Boolean;
begin
     Result:=IsWindowEnabled(Handle);
end;
procedure TWindow.SetEnabled(NewVal:Boolean);
begin
     EnableWindow(Handle,NewVal);
end;
function TWindow.GetWindowRect:TRect;
begin
     if Windows.GetWindowRect(Handle,Result) = False then raise Exception.Create('Win32 API Error: GetWindowRect');
end;
procedure TWindow.SetWindowRect(NewRect:TRect);
begin
     if Windows.MoveWindow(Handle,NewRect.Left,NewRect.Top,NewRect.Right-NewRect.Left,NewRect.Bottom-NewRect.Top,True)=False then raise Exception.Create('Win32 API Error: MoveWindow');
end;
function TWindow.ClientRect:TRect;
begin
     if Windows.GetClientRect(Handle,Result)=False then raise Exception.Create('Win32 API Error: GetClientRect');
end;
function TWindow.GetCoord(Index:Integer):Integer;
var rct:TRect;
begin
     rct:=Self.WindowRect;
     case Index of
     0:Result:=rct.Left;
     1:Result:=rct.Right;    //Not implemented in the class (commented out, remove comments in the class definition to make it work)
     2:Result:=rct.Top;
     3:Result:=rct.Bottom;   //Not implemented
     4:Result:=rct.Right-rct.Left;
     5:Result:=rct.Bottom-rct.Top;
     end;
end;
procedure TWindow.SetCoord(Index:Integer;NewVal:Integer);
var rct:TRect;
begin
     rct:=Self.WindowRect;
     case index of
     0:begin
        rct.Right:=rct.Right+NewVal-rct.Left;
        rct.Left:=NewVal;
     end;
     1:rct.Right:=NewVal;     //Not implemented
     2:begin
        rct.Bottom:=rct.Bottom+NewVal-rct.Top;
        rct.Top:=NewVal;
     end;
     3:rct.Bottom:=NewVal;    //Not implemented
     4:rct.Right:=NewVal+rct.Left;
     5:rct.Bottom:=NewVal+rct.Top;
     end;
     if Windows.MoveWindow(Handle,rct.Left,rct.Top,rct.Right-rct.Left,rct.Bottom-rct.Top,True)=False then raise Exception.Create('Win32 API Error: MoveWindow');
end;
function TWindow.GetVisible:Boolean;
begin
     Result:=IsWindowVisible(Handle);
end;
procedure TWindow.SetVisible(NewVal:Boolean);
begin
     if NewVal then Windows.ShowWindowAsync(Handle,SW_SHOWNA)
        else Windows.ShowWindowAsync(Handle,SW_HIDE);
end;
function TWindow.GetMinimized:Boolean;
begin
     Result:=IsIconic(Handle);
end;
procedure TWindow.SetMinimized(NewVal:Boolean);
begin
     if NewVal then Windows.ShowWindowAsync(Handle,SW_MINIMIZE)
        else if Self.Minimized then Windows.ShowWindowAsync(Handle,SW_SHOWNOACTIVATE);
end;
function TWindow.GetMaximized:Boolean;
begin
     Result:=IsZoomed(Handle);
end;
procedure TWindow.SetMaximized(NewVal:Boolean);
begin
     if NewVal then Windows.ShowWindowAsync(Handle,SW_MAXIMIZE)
        else if Self.Maximized then Windows.ShowWindowAsync(Handle,SW_SHOWNOACTIVATE);
end;
function TWindow.GetRestored:Boolean;
begin
     Result:=Not(Self.Maximized or Self.Minimized);
end;
//function TWindow.GetTopmost:Boolean;
//begin
  //I don't know how to do it !!!!!!!!!!!!!!!!!!!!!!
//end;
procedure TWindow.SetTopmost(NewVal:Boolean);
var TopMostFlag:HWND;
begin
     if NewVal=True then TopMostFlag:=HWND_TOPMOST else TopMostFlag:=HWND_NOTOPMOST;
     if Windows.SetWindowPos(Handle,TopMostFlag,0,0,0,0,SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE)=False
        then raise Exception.Create('Win32 API Error: SetWindowPos');
end;
procedure TWindow.SetRestored(NewVal:Boolean);
begin
     if NewVal then Windows.ShowWindowAsync(Handle,SW_RESTORE);
end;
function TWindow.GetMenu:HMenu;
begin
     Result:=Windows.GetMenu(Handle);
end;
procedure TWindow.SetMenu(NewMenu:HMenu);
begin
     if Windows.SetMenu(Handle,NewMenu) = False then raise Exception.Create('Win32 API Error: SetMenu');
end;
function TWindow.Move(Left,Top,Width,Height:Integer;Repaint:Boolean):Boolean;
begin
     Result:=Windows.MoveWindow(Handle,Left,Top,Width,Height,Repaint);
end;
function TWindow.IsChildOf(ParentHwnd:HWND):Boolean;
begin
     Result:=IsChild(ParentHwnd,Parent);
end;
function TWindow.ClienthDC:HDC;
begin
     Result:=Windows.GetDC(Handle);
end;
function TWindow.WindowHDC:HDC;
begin
     Result:=GetWindowDC(Handle);
end;
function TWindow.GetWindowFromDC(hDC:HDC):HWND;
begin
     Result:=Windows.WindowFromDC(hDC);
end;
procedure TWindow.Repaint;
begin
     Windows.UpdateWindow(Handle);
end;
function TWindow.GetWindowFromPoint(pt:TPoint):HWND;
begin
     Result:=Windows.WindowFromPoint(pt);
end;
function TWindow.ChildFromPoint(pt:TPoint):HWND;
begin
     Result:=Windows.ChildWindowFromPoint(Handle,pt);
end;
function TWindow.ChildFromPointEx(pt:TPoint;Flags:Cardinal):HWND;
begin
     Result:=Windows.ChildWindowFromPointEx(Handle,pt,Flags);
end;
function TWindow.ClientToScreen(pt:TPoint):TPoint;
var RetPt:TPoint;
begin
     RetPt:=pt;
     if Windows.ClientToScreen(Handle,RetPt)=False then
        raise Exception.Create('Win32 API Error: ScreenToClient')
     else Result:=RetPt;
end;
function TWindow.ScreenToClient(pt:TPoint):TPoint;
var RetPt:TPoint;
begin
     RetPt:=pt;
     if Windows.ScreenToClient(Handle,RetPt)=False then
        raise Exception.Create('Win32 API Error: ScreenToClient')
     else Result:=RetPt;
end;
function EnumWindowsProc(hHWND:HWND;lParam:Integer):Boolean;stdcall;
var
   ArrLen:Integer; cbText:Integer; lpText:PChar;
   bVisible,EnumVis,EnumUnvis:Boolean;
begin
     Result:=True;
     EnumVis:=(lParam and KTEW_ENUMVISIBLE)<>0;
     EnumUnVis:=(lParam and KTEW_ENUMINVISIBLE)<>0;
     bVisible:=IsWindowVisible(hHWND);
     if not((EnumVis and bVisible) or (EnumUnvis and (not bVisible))) then Exit;
     ArrLen:=Length(HwndArray);
     SetLength(HwndArray,ArrLen+1);
     HwndArray[ArrLen]:=hHWND;
     if (lParam and KTEW_GETTEXTS) <> 0 then begin  //Do we need to fill windowTexts array??
        cbText:=GetWindowTextLength(hHWND);
        lpText:=StrAlloc(cbText+1);
        SendMessage(hHWND,WM_GETTEXT,cbText+1,Integer(lpText));
        WindowTexts.Add(String(lpText));
     end;
     if (lParam and KTEW_GETCLASSES) <> 0 then begin  //Do we need to fill windowClasses array??
        lpText:=StrAlloc(MAX_PATH+1);
        GetClassName(hHWND,lpText,MAX_PATH+1);
        WindowClasses.Add(String(lpText));
     end;
end;
function TWindow.EnumWindows:THwndArray;
begin
     SetLength(HwndArray,0);
     if (FEnumFlags and KTEW_GETTEXTS) <> 0 then WindowTexts.Clear;
     if (FEnumFlags and KTEW_GETCLASSES) <> 0 then WindowClasses.Clear;
     Windows.EnumWindows(@EnumWindowsProc,FEnumFlags);//=False then
        //raise Exception.Create('Win32 API Error: EnumWindows');
     Result:=HwndArray;
end;
function TWindow.EnumChildWindows:THwndArray;
begin
     SetLength(HwndArray,0);
     if (FEnumFlags and KTEW_GETTEXTS) <> 0 then WindowTexts.Clear;
     if (FEnumFlags and KTEW_GETCLASSES) <> 0 then WindowClasses.Clear;
     Windows.EnumChildWindows(Handle,@EnumWindowsProc,FEnumFlags); //=False then
        //raise Exception.Create('Win32 API Error: EnumChildWindows');
     Result:=HwndArray;
end;
function TWindow.EnumThreadWindows(dwThreadID:Cardinal):THwndArray;
begin
     SetLength(HwndArray,0);
     if (FEnumFlags and KTEW_GETTEXTS) <> 0 then WindowTexts.Clear;
     if (FEnumFlags and KTEW_GETCLASSES) <> 0 then WindowClasses.Clear;
     Windows.EnumThreadWindows(dwThreadID,@EnumWindowsProc,FEnumFlags);//=False then
        //raise Exception.Create('Win32 API Error: EnumThreadWindows');
     Result:=HwndArray;
end;
function TWindow.ClassName:String;
var lpClass:PChar;
begin
     lpClass:=StrAlloc(MAX_PATH+1);
     if Windows.GetClassname(Handle,lpClass,MAX_PATH) = 0 then raise Exception.Create('Win32 API Error: GetClassName');
     Result:=String(lpClass);
end;
function TWindow.GetText:String;
var cbText:Integer; lpText:PChar;
begin
     cbText:=TextLength;
     lpText:=StrAlloc(cbText+1);
//GetWindowText will not work with Edit boxes in other applications
//(it is so written in the help) that's why we use WM_GETTEXT
     Windows.SendMessage(Handle,WM_GETTEXT,cbText+1,Integer(lpText));
     Result:=String(lpText);
end;
function TWindow.GetIcon(Index:Integer):HIcon;
begin
     Result:=Windows.SendMessage(Handle,WM_GETICON,Index,0);
end;
procedure TWindow.SetIcon(Index:Integer;NewIcon:HIcon);
begin
     Windows.SendMessage(Handle,WM_SETICON,Index,Integer(NewIcon));
end;
function TWindow.PostMessage(Msg:Cardinal;wParam,lParam:Integer):Boolean;
begin
     Result:=Windows.PostMessage(Handle,Msg,wParam,lParam);
end;
function TWindow.SendMessage(Msg:Cardinal;wParam,lParam:Integer):Integer;
begin
     Result:=Windows.SendMessage(Handle,Msg,wParam,lParam);
end;
function TWindow.CreateWindowEx;
begin
     Result:=Windows.CreateWindowEx(dwExStyle,lpClassName,lpWindowName,
           dwStyle,X,Y,nWidth,nHeight,hWndParent,
           hMenu,hInstance,lpParam);
end;
function TWindow.ListItemCount:Integer;
begin
     Result:=Windows.SendMessage(Handle,CB_GETCOUNT,0,0);
end;
function TWindow.ListItems:TStringList;  //Returns a list of items in a ListBox or a ComboBox;
var SL:TStringList;cItems,i,cbText:Integer;lpText:PChar;
begin
     SL:=TStringList.Create;
     cItems:=Windows.SendMessage(Handle,CB_GETCOUNT,0,0);
     for i:=0 to cItems-1 do begin
         cbText:=Windows.SendMessage(Handle,CB_GETLBTEXTLEN,i,0);
         lpText:=StrAlloc(cbText+1);
         Windows.SendMessage(Handle,CB_GETLBTEXT,i,Integer(lpText));
         SL.Add(String(lpText));
     end;
     Result:=SL;
end;
//*************************TWindowClass*************************
constructor TWindowClass.Create(NewHwnd:HWND);
begin
     inherited Create;
     Handle:=NewHWND;
end;
function TWindowClass.GetCLng(Index:Integer):Cardinal;
begin
     Result:=GetClassLong(Handle,Index);
end;
procedure TWindowClass.SetCLng(Index:Integer;NewVal:Cardinal);
begin
     if SetClassLong(Handle,Index,Integer(NewVal))=0 then raise Exception.Create('Win32 API Error: SetClassLong');
end;
function TWindowClass.GetWindowProc:TWindowProc;
begin
     @Result:=Pointer(GetClassLong(Handle,GCL_WNDPROC));
end;
procedure TWindowClass.SetWindowProc(NewProc:TWindowProc);
begin
     if SetClassLong(Handle,GCL_WNDPROC,Integer(@NewProc))=0 then raise Exception.Create('Win32 API Error: SetClassLong(GWL_WNDPROC)');
end;
function TWindowClass.Name:String;
begin
     Result:=StringOfChar(' ',MAX_PATH+1);
     if Windows.GetClassName(Handle,PChar(Result),MAX_PATH) = 0 then raise Exception.Create('Win32 API Error: GetClassName');
     if Pos(#0,Result)<>0 then Result:=Copy(Result,1,Pos(#0,Result)-1);
end;
function TWindowClass.RegisterClassEx(WndClass:TWndClassEx):ATOM;
begin
     Result:=Windows.RegisterClassEx(WndClass);
end;
function TWindowClass.UnregisterClass(lpClassName:PChar;hInstanc:HINST):Boolean;
begin
     Result:=Windows.UnregisterClass(lpClassName,hInstanc);
end;

//These somehow didn't wish to work

{function TWindowClass.GetCMenu:String;
begin
     Result:=String(PChar(GetClassLong(Handle,GCL_MENUNAME)));
end;
procedure TWindowClass.SetCMenu(NewVal:String);
begin
     if SetClassLong(Handle,GCL_MENUNAME,Integer(PChar(NewVal)))=0 then raise Exception.Create('Win32 API Error: SetClassLong');
end;   }
{function TWindowClass.ClassInfo:tagWNDCLASSEXA;
     //This one duzn't work, I don't know why
var ClsName:PChar;bRes:Boolean;
begin
     Result.cbSize:=SizeOf(Result);
     ClsName:=StrAlloc(MAX_PATH+1);
     if GetClassName(Handle,ClsName,MAX_PATH+1)=0 then raise Exception.Create('Win32 API Error: GetClassName');
     Result.lpszClassName:=ClsName;
     Result.hInstance:=GetWindowLong(Handle,GWL_HINSTANCE);
     if Result.hInstance=0 then raise Exception.Create('Win32 API Error: GetWindowLong');
     bRes:=GetClassInfoEx(Result.hInstance,ClsName,Result);
     if bRes=False then raise Exception.Create('Win32 API Error: GetClassInfoEx');
end; }
end.

//1999-2000 Konstantin Tretyakov

